/*
 *  File Name: RCSTRING.H
 *
 *  Permission to use, copy, modify, and distribute this software
 *  for NON-COMMERCIAL purposes and without fee is hereby granted
 *  provided that this notice and the copyright notice below appear
 *  in all copies.
 *
 *  This software is adapted from the text
 *
 *      "Inside the Object Model: The Sensible Use of C++"
 *              by David M. Papurt,
 *                  published by SIGS Books
 *                      ISBN 1-884842-05-4
 *
 *  DAVID PAPURT MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 *  OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 *  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 *  PURPOSE, OR NON-INFRINGEMENT. DAVID PAPURT SHALL NOT BE LIABLE FOR
 *  ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING, OR
 *  DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 *  THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 *  CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 *  PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 *  NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 *  SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 *  SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 *  PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  DAVID PAPURT
 *  SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 *  HIGH RISK ACTIVITIES.
 *
 *  Copyright (c) 1995, 1996 by David M. Papurt. All Rights Reserved.
 *
 */

#ifndef RCSTRING_H
#define RCSTRING_H

class StringRep
{
	int c;		// reference count
	char * p;	// useful information

	friend class String;
		// private data accessible only from
		// class String member functions
};

class String
{
	StringRep * r;
		// Representation Invariant:
		//   r addresses a StringRep object
		//
		//   r->c is count of r pointers
		//   currently addressing the StringRep
		//   object *r
		//
		//   r->c > 0
		//
		//   r->p addresses a 0 terminated
		//   character array allocated on the
		//   heap for r->p's sole use

	const char * rp() const { return r->p; }

public:
	String( char * );
	~String();
	String & operator=( const String & );
	String( const String & );

	friend int operator==( const String & L,
                          const String & R );

};

#endif
  // RCSTRING_H