/*
 *  File Name: VLIST.CPP
 *
 *  Permission to use, copy, modify, and distribute this software
 *  for NON-COMMERCIAL purposes and without fee is hereby granted
 *  provided that this notice and the copyright notice below appear
 *  in all copies.
 *
 *  This software is adapted from the text
 *
 *      "Inside the Object Model: The Sensible Use of C++"
 *              by David M. Papurt,
 *                  published by SIGS Books
 *                      ISBN 1-884842-05-4
 *
 *  DAVID PAPURT MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 *  OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 *  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 *  PURPOSE, OR NON-INFRINGEMENT. DAVID PAPURT SHALL NOT BE LIABLE FOR
 *  ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING, OR
 *  DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 *  THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 *  CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 *  PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 *  NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 *  SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 *  SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 *  PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  DAVID PAPURT
 *  SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 *  HIGH RISK ACTIVITIES.
 *
 *  Copyright (c) 1995, 1996 by David M. Papurt. All Rights Reserved.
 *
 */

#include <vlist.h>
#include <iostream.h>

VLink::VLink( void * d, VLink * n )
: next(n),
  data(d)
{}

VList::VList()
: c(0),
  head(0)
{}

VList::~VList()
{
	if( head )
	  cerr << "Terminating VList not empty\n";
}

void VList::insert( void * p )
{
	head = new VLink( p, head );
}

void * VList::remove()
{
	if ( head )
	{
		void * r = head->data;
		VLink * t = head;
		head = head->next;
		delete t;
		return r;
	}
	else
		return 0;
}

void * VList::get()
{
	if ( head )
		return head->data;
	else
		return 0;
}

int VList::isIn( void * p )
{
	void * q;
	reset();
	while(  (q=next()) != 0  )
		if ( q == p )
			return 1;
	return 0;
}

void * VList::remove( void * p )
{
	VLink * curr = head;
	VLink * prev = 0;

	while( curr )
	{
		if ( curr->data == p )
		{
			if ( prev )
				prev->next = curr->next;
			else
				head = curr->next;

			delete curr;
			return p;
		}
		prev = curr;
		curr = curr->next;
	}
	return 0;
}

void VList::reset()
{
	c = head;
}

void * VList::next()
{
	if ( c )
	{
		void * r = c->data;
		c = c->next;
		return r;
	}
	else
		return 0;
}

