/*
 *  File Name: A1_M_BIA.H
 *
 *  Permission to use, copy, modify, and distribute this software
 *  for NON-COMMERCIAL purposes and without fee is hereby granted
 *  provided that this notice and the copyright notice below appear
 *  in all copies.
 *
 *  This software is adapted from the text
 *
 *      "Inside the Object Model: The Sensible Use of C++"
 *              by David M. Papurt,
 *                  published by SIGS Books
 *                      ISBN 1-884842-05-4
 *
 *  DAVID PAPURT MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 *  OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 *  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 *  PURPOSE, OR NON-INFRINGEMENT. DAVID PAPURT SHALL NOT BE LIABLE FOR
 *  ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING, OR
 *  DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 *  THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 *  CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 *  PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 *  NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 *  SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 *  SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 *  PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  DAVID PAPURT
 *  SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 *  HIGH RISK ACTIVITIES.
 *
 *  Copyright (c) 1995, 1996 by David M. Papurt. All Rights Reserved.
 *
 */

#ifndef A1_M_BIA_H
#define A1_M_BIA_H

#include <tlist.cpp>

template <class LB,class AB,class RB>
class LT1toM;

template <class LB,class AB,class RB>
class RT1toM;

template <class LB,class AB,class RB>
class AT1toM : public AB
{
	LT1toM<LB,AB,RB> * l;
	RT1toM<LB,AB,RB> * r;

	AT1toM( const AT1toM & );
	AT1toM & operator=(const AT1toM &);
		// COPY FUNCTIONS NOT IMPLEMENTED!

public:
	AT1toM( LT1toM<LB,AB,RB> &,
	        RT1toM<LB,AB,RB> & );

	~AT1toM();

	LT1toM<LB,AB,RB> * left() { return l; }

	RT1toM<LB,AB,RB> * right() { return r; }
};

template <class LB,class AB,class RB>
class LT1toM : public LB
{
	List< AT1toM<LB,AB,RB> > list;

	void insert( AT1toM<LB,AB,RB> * p )
		{ list.insert( p ); }

	AT1toM<LB,AB,RB> * remove(
							AT1toM<LB,AB,RB> * p )
		{ return list.remove(p); }

	friend class AT1toM<LB,AB,RB>;

public:
	LT1toM() : LB(), list() {}

	~LT1toM();

	void reset() { list.reset(); }
	AT1toM<LB,AB,RB> * assoc()
		{ return list.next(); }
	RT1toM<LB,AB,RB> * right()
	{
		AT1toM<LB,AB,RB> * q = list.next();
		return q ? q->right() : 0;
	}
};

template <class LB,class AB,class RB>
class RT1toM : public RB
{
	AT1toM<LB,AB,RB> * a;

	void set(AT1toM<LB,AB,RB> * p) { a = p; }

	friend class AT1toM<LB,AB,RB>;

public:
	RT1toM() : RB(), a(0) {}

	~RT1toM() { delete a; }

	AT1toM<LB,AB,RB> * assoc() { return a; }
	LT1toM<LB,AB,RB> * left()
		{ return a ? a->left() : 0; }
};

#endif
  // A1_M_BIA_H
